import pprint
import logging

from checkbox.contrib import bpickle
from checkbox.contrib.persist import Persist, MemoryBackend

from checkbox.plugin import Plugin
from checkbox.properties import Int, Path
from checkbox.message import ANCIENT, BROKEN, MessageStore, got_next_sequence


class CertifyExchange(Plugin):

    # Directory where messages are stored
    store_directory = Path(default="%(checkbox_data)s/messages")

    # Maximum number of messages per directory
    store_directory_size = Int(default=1000)

    # Maximum number of messages per request
    max_messages = Int(default=20)

    @property
    def persist(self):
        if self._persist is None:
            self._persist = Persist(backend=MemoryBackend())

        return self._persist.root_at("certify_exchange")

    @property
    def message_store(self):
        if self._message_store is None:
            self._message_store = MessageStore(self.persist,
                self.store_directory, self.store_directory_size)

        return self._message_store

    def register(self, manager):
        super(CertifyExchange, self).register(manager)

        self._count_messages = 0
        self._message_store = None
        self._persist = None

        for (rt, rh) in [
             ("begin-persist", self.begin_persist),
             ("certify-exchange", self.certify_exchange),
             ("exchange-error", self.exchange_error),
             ("message-send", self.message_send),
             ("transport-receive", self.transport_receive)]:
            self._manager.reactor.call_on(rt, rh)

    def begin_persist(self, persist):
        self._persist = persist

    def certify_exchange(self):
        message_store = self.message_store

        self._error = None
        while self._error is None:
            # Run at least once to retrieve messages too
            messages = message_store.get_pending_messages(self.max_messages)
            self._count_messages = len(messages)

            payload = {
                "messages": messages,
                "sequence": message_store.get_sequence(),
                "total-messages": message_store.count_pending_messages()}
            request = bpickle.dumps(payload)
            self._manager.reactor.fire("transport-send", request)

            if not message_store.count_pending_messages():
                break

    def exchange_error(self, error):
        self._error = error

    def message_send(self, message):
        self.message_store.add(message)
        self.message_store.commit()

    def transport_receive(self, response):
        message_store = self.message_store
        payload = bpickle.loads(response)

        next_sequence = payload.get("next-sequence")
        if next_sequence is None:
            next_sequence = message_store.get_sequence()
            next_sequence += self._count_messages

        state = got_next_sequence(message_store, next_sequence)
        message_store.commit()
        if state == ANCIENT:
            # The server has probably lost some data we sent it. The
            # slate has been wiped clean (by got_next_sequence), now
            # let's fire an event to tell all the plugins that they
            # ought to generate new messages so the server gets some
            # up-to-date data.
            logging.info("Server asked for ancient data.")

        for message in payload.get("messages", ()):
            if logging.getLogger().getEffectiveLevel() <= logging.DEBUG:
                logging.debug("Response message:\n%s",
                              pprint.pformat(message))

            if message["type"] == "error":
                message_store.add_pending_flags(BROKEN)
                self._manager.reactor.fire("exchange-error",
                    message["message"])

            self._manager.reactor.fire("message-receive", message)
            message_store.commit()


factory = CertifyExchange
