import time
import pprint
import logging

from checkbox.lib.log import format_delta
from checkbox.lib.transport import HTTPTransport

from checkbox.plugin import Plugin
from checkbox.properties import Int, String


class CertifyTransport(Plugin):

    # URL where to send submissions.
    transport_url = String(default="https://certification.canonical.com/message")

    # Timeout value for each submission.
    timeout = Int(default=360)

    # Header to identify the message interface version.
    message_api_header = String(default="X-Message-API")

    # Version of the certification message interface.
    message_api_version = String(default="1.0")

    # Header to identify the hardware ID.
    hardware_id_header = String(default="X-Hardware-ID")

    # Header to identify the submission ID.
    submission_id_header = String(default="X-Submission-ID")

    def register(self, manager):
        super(CertifyTransport, self).register(manager)

        self._headers = {self.message_api_header: self.message_api_version}

        for (rt, rh) in [
             ("begin-persist", self.begin_persist),
             ("gather", self.gather),
             ("transport-send", self.transport_send),
             ("report-client", self.report_client),
             ("report-hardware-id", self.report_hardware_id),
             ("report-submission", self.report_submission)]:
            self._manager.reactor.call_on(rt, rh)

    def begin_persist(self, persist):
        self.persist = persist.root_at("certify-transpot")
        cookie = self.persist.get("cookie")
        if cookie is not None:
            self._headers["Cookie"] = cookie

    def gather(self):
        self._manager.reactor.fire("transport-url", self.transport_url)

    def report_client(self, message):
        user_agent = "%s/%s" % (message["name"], message["version"])
        self._headers["User-Agent"] = user_agent

    def report_hardware_id(self, hardware_id):
        self._headers[self.hardware_id_header] = hardware_id

    def report_submission(self, message):
        self._headers[self.submission_id_header] = message["secure_id"]

    def transport_send(self, request):
        transport = HTTPTransport(self.transport_url)

        start_time = time.time()
        try:
            response = transport.exchange(request, self._headers, self.timeout)
        except Exception as error:
            self._manager.reactor.fire("exchange-error", error)
            return

        end_time = time.time()

        if not response:
            self._manager.reactor.fire("exchange-error", """\
Error contacting the server: %s""" % self.transport_url)
        elif response.status != 200:
            self._manager.reactor.fire("exchange-error", """\
Server returned unexpected status: %d""" % response.status)
        else:
            if logging.getLogger().getEffectiveLevel() <= logging.DEBUG:
                logging.debug("Response headers:\n%s",
                              pprint.pformat(response.getheaders()))

            cookie = response.getheader("set-cookie")
            if cookie is not None:
                self.persist.set("cookie", cookie)
                self._headers["Cookie"] = cookie

            text = response.read()
            logging.info("Sent %d bytes and received %d bytes in %s.",
                len(request), len(text), format_delta(end_time - start_time))

            self._manager.reactor.fire("transport-receive", text)


factory = CertifyTransport
